@@ -1326,15 +1326,19 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    calcarc(newarc);
 
+   areawin->save.x = newpos.x;
+   areawin->save.y = newpos.y;
+
    XcSetXORFg(newarc->color, BACKGROUND);
    UDrawArc(newarc, xobjs.pagelist[areawin->page]->wirewidth);
    UDrawXLine(newpos, newarc->position);
    printpos(newpos.x, newpos.y);
 
-   areawin->save.x = newpos.x;
-   areawin->save.y = newpos.y;
-
    flusharea();
+#ifdef HAVE_CAIRO
+   if (areawin->redraw_needed)
+     drawarea(NULL, NULL, NULL);
+#endif /* HAVE_CAIRO */
 }
 
 /*--------------------------------------*/
@@ -1421,6 +1425,10 @@ void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
    areawin->save.y = newpos.y;
 
    flusharea();
+#ifdef HAVE_CAIRO
+   if (areawin->redraw_needed)
+     drawarea(NULL, NULL, NULL);
+#endif /* HAVE_CAIRO */
 }
 
 /*----------------------------------------------------------------------*/
@@ -1465,6 +1473,10 @@ void trackwire(xcWidget w, caddr_t clientdata, caddr_t calldata)
    }
 
    flusharea();
+#ifdef HAVE_CAIRO
+   if (areawin->redraw_needed)
+     drawarea(NULL, NULL, NULL);
+#endif /* HAVE_CAIRO */
 }
 
 /*--------------------------*/
@@ -1688,6 +1700,10 @@ void trackelement(xcWidget w, caddr_t clientdata, caddr_t calldata)
    areawin->save.y = newpos.y;
 
    flusharea();
+#ifdef HAVE_CAIRO
+   if (areawin->redraw_needed)
+     drawarea(NULL, NULL, NULL);
+#endif /* HAVE_CAIRO */
 }
 
 /*-------------------------------------------------*/
@@ -1829,7 +1845,7 @@ void unjoin()
 
    for (selectobj = areawin->selectlist; selectobj < areawin->selectlist
         + areawin->selects; selectobj++) {
-      XSetForeground(dpy, areawin->gc, BACKGROUND);
+      SetForeground(dpy, areawin->gc, BACKGROUND);
       if (SELECTTYPE(selectobj) == PATH) {
          oldpath = SELTOPATH(selectobj);
 
