@@ -296,6 +296,9 @@ menustruct FileMenu[] = {
 	{"Read Xcircuit File", action(getfile, Number(NORMAL))},
 	{"Import Xcircuit PS", action(getfile, Number(IMPORT))},
 	{"Import background PS", action(getfile, Number(PSBKGROUND))},
+#ifdef HAVE_CAIRO
+	{"Import graphic image", action(getfile, Number(IMPORTGRAPHIC))},
+#endif
 #ifdef ASG
 	{"Import SPICE deck", action(getfile, Number(IMPORTSPICE))},
 #endif
@@ -304,7 +307,7 @@ menustruct FileMenu[] = {
 	{" ", noaction},
 	{"Add To Library", action(getlib, NULL)},
 	{"Load New Library", action(getuserlib, NULL)},
-	{"Save Library", action(savelibpopup, Number(0))},
+	{"Save Library", action(savelibpopup, NULL)},
 	{" ", noaction},
         {"Clear Page", action(resetbutton, Number(0))},
 	{" ", noaction},
