@@ -158,9 +158,14 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
+#ifdef HAVE_CAIRO
+   cairo_save(areawin->cr);
+   cairo_reset_clip(areawin->cr);
+#else
    // Note:  Setting areawin->clipped to -1 prevents the clipmask from being
    // applied to elements, 
    areawin->clipped = -1;
+#endif /* HAVE_CAIRO */
 
    switch (ELEMENTTYPE(*(curobj->plist + instance))) {
       case ARC:
@@ -186,7 +191,11 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
          UDrawGraphic((graphicptr)elementptr);
 	 break;
    }
+#ifdef HAVE_CAIRO
+   cairo_restore(areawin->cr);
+#else
    areawin->clipped = 0;
+#endif /* HAVE_CAIRO */
 }
 
 /*-------------------------------------------------*/
@@ -200,16 +209,26 @@ void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 
    if (*newselect >= curobj->parts) return;	// Safety check
 
+#ifndef HAVE_CAIRO
    XcSetFunction(GXcopy);
    XcSetForeground(BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
+#endif /* !HAVE_CAIRO */
 
+#ifdef HAVE_CAIRO
+   XcSetForeground(SELECTCOLOR);
+#else
    XcSetFunction(GXxor);
    XcSetForeground(SELECTCOLOR ^ BACKGROUND);
+#endif /* HAVE_CAIRO */
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
+#ifdef HAVE_CAIRO
+   SetForeground(dpy, areawin->gc, AUXCOLOR);
+#else
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+#endif /* HAVE_CAIRO */
    indicateparams(*(curobj->plist + *newselect));
 
    SetForeground(dpy, areawin->gc, areawin->gccolor);
