@@ -5,6 +5,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #ifndef XC_WIN32
 #include <unistd.h>
@@ -229,13 +230,17 @@ void SVGCreateImages(int page)
        fname = tmpnam(NULL);
        ppf = fopen(fname, "w");
        if (ppf != NULL) {
-	  fprintf(ppf, "P6 %d %d 255\n", img->image->width, img->image->height);
-	  for (y = 0; y < img->image->height; y++) {
-	     for (x = 0; x < img->image->width; x++) {
-	        pixel.i = XGetPixel(img->image, x, y);
-		fwrite(&pixel.b[2], 1, 1, ppf);
-		fwrite(&pixel.b[1], 1, 1, ppf);
-		fwrite(&pixel.b[0], 1, 1, ppf);
+    	  int width, height;
+	  width = xcImageGetWidth(img->image);
+	  height = xcImageGetWidth(img->image);
+	  fprintf(ppf, "P6 %d %d 255\n", width, height);
+	  for (y = 0; y < height; y++) {
+	     for (x = 0; x < width; x++) {
+		u_char r, g, b;
+		xcImageGetPixel(img->image, x, y, &r, &g, &b);
+		fwrite(&r, 1, 1, ppf);
+		fwrite(&g, 1, 1, ppf);
+		fwrite(&b, 1, 1, ppf);
 	     }
 	  }
        }
@@ -257,7 +262,7 @@ void SVGCreateImages(int page)
        waitpid(pid, NULL, 0);
        unlink(fname);
 #else
-       _spawnl(_P_WAIT, GM_EXC, GM_EXEC, "convert", fname, outname, NULL);
+       _spawnl(_P_WAIT, GM_EXEC, GM_EXEC, "convert", fname, outname, NULL);
        _unlink(fname);
 #endif
        Fprintf(stdout, "Generated standalone PNG image file %s\n", outname);
@@ -275,6 +280,8 @@ void SVGDrawGraphic(graphicptr gp)
     char outname[128], *pptr;
     float tscale;
     int rotation;
+    int width = xcImageGetWidth(gp->source);
+    int height = xcImageGetHeight(gp->source);
 
     for (i = 0; i < xobjs.images; i++) {
        img = xobjs.imagelist + i;
@@ -291,8 +298,8 @@ void SVGDrawGraphic(graphicptr gp)
 
     UPushCTM();
     UPreMultCTM(DCTM, gp->position, gp->scale, gp->rotation);
-    corner.x = -(gp->source->width >> 1);
-    corner.y = (gp->source->height >> 1);
+    corner.x = -(width >> 1);
+    corner.y = (height >> 1);
     UTransformbyCTM(DCTM, &corner, &ppt, 1);
     UPopCTM();
 
@@ -303,8 +310,7 @@ void SVGDrawGraphic(graphicptr gp)
     
     fprintf(svgf, "<image transform=\"translate(%d,%d) scale(%g) rotate(%d)\"\n",
 			ppt.x, ppt.y, tscale, rotation);
-    fprintf(svgf, "  width=\"%dpx\" height=\"%dpx\"",
-			gp->source->width, gp->source->height);
+    fprintf(svgf, "  width=\"%dpx\" height=\"%dpx\"", width, height);
     fprintf(svgf, " xlink:href=\"%s\">\n", outname);
     fprintf(svgf, "</image>\n");
 }
