@@ -23,12 +23,25 @@ typedef unsigned long long	u_long;
 #endif
 #endif
 
+#ifdef HAVE_CAIRO
+#include <cairo/cairo.h>
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Graphics functions defined for X11 (i.e., not OpenGL)		*/
 /*----------------------------------------------------------------------*/
 
 #ifndef OPENGL
 
+#ifdef HAVE_CAIRO
+
+#define SetForeground(display, gc, fg) xc_cairo_set_color(fg)
+#define SetFunction(display, gc, function)
+typedef cairo_surface_t xcImage;
+
+#else /* HAVE_CAIRO */
+
+typedef XImage xcImage;
 #define DrawLine	XDrawLine
 #define DrawLines	XDrawLines
 #define DrawPoint	XDrawPoint
@@ -43,6 +56,9 @@ typedef unsigned long long	u_long;
 #define SetDashes	XSetDashes
 #define SetFillStyle	XSetFillStyle
 #define SetStipple(a,b,c) XSetStipple(a,b,STIPPLE[c])
+
+#endif /* HAVE_CAIRO */
+
 #define flusharea()	
 
 #else
@@ -358,6 +374,9 @@ enum editmode {
 enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #ifdef ASG
 	IMPORTSPICE,
+#endif
+#ifdef HAVE_CAIRO
+	IMPORTGRAPHIC,
 #endif
 	LOAD_MODES
 };
@@ -775,12 +794,14 @@ typedef struct {
    XPoint	  position;
    short	  rotation;
    float	  scale;
-   XImage	  *source;	/* source data */
-   XImage	  *target;	/* target (scaled) data */
+   xcImage	  *source;	/* source data */
+#ifndef HAVE_CAIRO
+   xcImage	  *target;	/* target (scaled) data */
    short	  trot;		/* target rotation */
    float	  tscale;	/* target scale (0 = uninitialized) */
    Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
    Boolean	  valid;	/* does target need to be regenerated? */
+#endif /* HAVE_CAIRO */
 } graphic, *graphicptr;
 
 /*----------------------------------------------------------------------*/
@@ -1202,10 +1223,6 @@ typedef struct {
 #define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
 	areawin->gc, FOREGROUND); else SetForeground(dpy, areawin->gc, a)
 
-#define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
-	a ^ b)
-
 #define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
 	areawin->gctype = z
 
@@ -1215,9 +1232,22 @@ typedef struct {
 #define XcSetForeground(z) SetForeground(dpy, areawin->gc, z); \
 	areawin->gccolor = z
 
-#define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
+#ifdef HAVE_CAIRO
+
+   #define XSetXORFg(a,b) XTopSetForeground(a)
+   #define XcSetXORFg(y,z) XcTopSetForeground(y)
+
+#else /* HAVE_CAIRO */
+
+   #define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
+        areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
+        a ^ b)
+
+   #define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
+#endif /* HAVE_CAIRO */
+
 /*----------------------------------------------------------------------*/
 /* Structure for maintaining list of colors				*/
 /*----------------------------------------------------------------------*/
@@ -1247,6 +1277,10 @@ typedef struct {
    float scale;
    u_short flags;
    objectptr *encoding;
+#ifdef HAVE_CAIRO
+   const char **utf8encoding;
+   cairo_font_face_t *font_face;
+#endif /* HAVE_CAIRO */
 } fontinfo;
 
 /*----------------------------------------------------------------------*/
@@ -1365,7 +1399,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   XImage	*image;
+   xcImage	*image;
    int		refcount;
    char		*filename;
 } Imagedata;
@@ -1386,13 +1420,20 @@ typedef struct _windowdata {
    xcWidget	scrollbarh, scrollbarv;
    Window	window;
    GC 		gc;
+#ifndef HAVE_CAIRO
    Pixmap	clipmask;
    Pixmap	pbuf;		/* clipmask buffer for hierarchical clipping */
    char		clipped;
    GC		cmgc;
+#endif /* !HAVE_CAIRO */
    int		gccolor, gctype;
    xcIntervalId time_id;
-
+   Boolean	redraw_needed;
+   Boolean	redraw_ongoing;
+#ifdef HAVE_CAIRO
+   cairo_surface_t *surface;
+   cairo_t	*cr;		/* cairo_drawing context */
+#endif /* HAVE_CAIRO */
    /* global page parameters */
    short	width, height;
    short	page;
