@@ -41,6 +41,9 @@
 #include "Xw/Toggle.h"
 #endif
 
+#ifdef HAVE_CAIRO
+#include <cairo/cairo-xlib.h>
+#endif
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
 /*----------------------------------------------------------------------*/
@@ -66,8 +69,8 @@
 /* Global Variable definitions						*/
 /*----------------------------------------------------------------------*/
 
-short	 popups;      /* total number of popup widgets on the screen */
-int	 pressmode;   /* Whether we are in a press & hold state */
+extern short popups;	/* total number of popup widgets on the screen */
+extern int pressmode;	/* Whether we are in a press & hold state */
 
 xcWidget   top;
 xcWidget   message1, message2, message3, toolbar, overlay = NULL;
@@ -1960,6 +1963,11 @@ XCWindowData *GUI_init(int argc, char *argv[])
    XtAddEventHandler(top, NoEventMask, TRUE,
 	(XtEventHandler)clientmessagehandler, NULL);
 
+   /* Set the area widget width and height, center userspace (0, 0) on screen */
+   XtSetArg(wargs[0], XtNwidth, &newwin->width);
+   XtSetArg(wargs[1], XtNheight, &newwin->height);
+   XtGetValues(newwin->area, wargs, 2);
+
    /*---------------------------------------------------*/
    /* Define basic display variables 			*/
    /* Redefine win to be just the drawing area window   */
@@ -1978,11 +1986,19 @@ XCWindowData *GUI_init(int argc, char *argv[])
    newwin->gc = XCreateGC(dpy, newwin->window,
 		GCForeground | GCBackground | GCFont, &values);
 
-   /* set the area widget width and height, center userspace (0,0) on screen */
+#ifdef HAVE_CAIRO
+   newwin->surface = cairo_xlib_surface_create(dpy, newwin->window,
+         DefaultVisual(dpy, 0), newwin->width, newwin->height);
+   newwin->cr = cairo_create(newwin->surface);
+#else
+   newwin->clipmask = XCreatePixmap(dpy, newwin->window, newwin->width,
+		newwin->height, 1); 
 
-   XtSetArg(wargs[0], XtNwidth, &newwin->width);
-   XtSetArg(wargs[1], XtNheight, &newwin->height);
-   XtGetValues(newwin->area, wargs, 2);
+   values.foreground = 0;
+   values.background = 0;
+   newwin->cmgc = XCreateGC(dpy, newwin->clipmask, GCForeground
+		| GCBackground, &values);
+#endif /* HAVE_CAIRO */
 
    return newwin;
 }
@@ -2071,6 +2087,7 @@ int main(int argc, char **argv)
       findcrashfiles();
    }
 
+   xobjs.suspend = -1;
    return local_xloop();   /* No return---exit through quit() callback */
 }
 
